@@ -32,7 +32,6 @@
 
 #include "gnome-print-i18n.h"
 #include "gnome-print-private.h"
-#include "gpa-private.h"
 
 #include "gnome-print-transport.h"
 #include "gnome-print-ps2.h"
@@ -44,7 +43,7 @@ static void gnome_print_context_init (GnomePrintContext *ctx);
 
 static void gnome_print_context_finalize (GObject *object);
 
-static GnomePrintContext *gnome_print_context_create (gpointer get_type, GPANode *config);
+static GnomePrintContext *gnome_print_context_create (gpointer get_type, GnomePrintConfig *config);
 
 static GObjectClass *parent_class = NULL;
 
@@ -103,8 +102,7 @@ gnome_print_context_finalize (GObject *object)
 	}
 
 	if (ctx->config) {
-		gpa_node_unref (ctx->config);
-		ctx->config = NULL;
+		ctx->config = gnome_print_config_unref (ctx->config);
 	}
 
 	gp_gc_unref (ctx->gc);
@@ -113,17 +111,15 @@ gnome_print_context_finalize (GObject *object)
 }
 
 gint
-gnome_print_context_construct (GnomePrintContext *ctx, GPANode *config)
+gnome_print_context_construct (GnomePrintContext *ctx, GnomePrintConfig *config)
 {
 	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (config != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GPA_IS_NODE (config), GNOME_PRINT_ERROR_UNKNOWN);
 
 	g_return_val_if_fail (ctx->config == NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
-	ctx->config = config;
-	gpa_node_ref (ctx->config);
+	ctx->config = gnome_print_config_ref (config);
 
 	if (GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->construct)
 		GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->construct (ctx);
@@ -152,7 +148,7 @@ gnome_print_context_create_transport (GnomePrintContext *ctx)
  * @ctx: A #GnomePrintContext
  * @name: Name of the page
  *
- * starts new output page with @name. Naming is used for interactive
+ * Starts new output page with @name. Naming is used for interactive
  * contexts like #GnomePrintPreview and Document Structuring Convention
  * conformant PostScript output.
  * This function has to be called before any drawing methods and immediately
@@ -215,7 +211,7 @@ gnome_print_showpage (GnomePrintContext *ctx)
 	ret = GNOME_PRINT_OK;
 
 	if (GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->showpage)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->showpage (ctx);
+		ret = GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->showpage (ctx);
 
 	ctx->haspage = FALSE;
 
@@ -256,11 +252,11 @@ gnome_print_gsave (GnomePrintContext *ctx)
 
 	ret = GNOME_PRINT_OK;
 
+	gp_gc_gsave (ctx->gc);
+
 	if (GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->gsave)
 		ret = GNOME_PRINT_CONTEXT_GET_CLASS (ctx)->gsave (ctx);
 
-	gp_gc_gsave (ctx->gc);
-
 	return ret;
 }
 
@@ -508,36 +504,39 @@ GnomePrintContext *
 gnome_print_context_new (GnomePrintConfig *config)
 {
 	GnomePrintContext *ctx;
-	GPANode *cfgnode;
 	guchar *drivername;
 
 	g_return_val_if_fail (config != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (config), NULL);
 
-	cfgnode = GPA_NODE (config);
-
-	drivername = gpa_node_get_path_value (cfgnode, "Settings.Engine.Backend.Driver");
+	drivername = gnome_print_config_get (config, "Settings.Engine.Backend.Driver");
 	g_return_val_if_fail (drivername != NULL, NULL);
 
 	ctx = NULL;
 
 	if (!strcmp (drivername, "gnome-print-ps")) {
 		GnomePrintContext *ps;
-		ps = gnome_print_ps2_new (cfgnode);
+		ps = gnome_print_ps2_new (config);
 		ctx = gnome_print_frgba_new (ps);
 		g_object_unref (G_OBJECT (ps));
 	} else {
 		guchar *modulename;
-		modulename = gpa_node_get_path_value (cfgnode, "Settings.Engine.Backend.Driver.Module");
+		modulename = gnome_print_config_get (config, "Settings.Engine.Backend.Driver.Module");
 		if (modulename) {
+			static GHashTable *modules = NULL;
 			GModule *module;
-			gchar *path;
-			path = g_module_build_path (GNOME_PRINT_LIBDIR "/drivers", modulename);
-			module = g_module_open (path, G_MODULE_BIND_LAZY);
+			if (!modules) modules = g_hash_table_new (g_str_hash, g_str_equal);
+			module = g_hash_table_lookup (modules, modulename);
+			if (!module) {
+				gchar *path;
+				path = g_module_build_path (GNOME_PRINT_LIBDIR "/drivers", modulename);
+				module = g_module_open (path, G_MODULE_BIND_LAZY);
+				if (module) g_hash_table_insert (modules, g_strdup (modulename), module);
+				g_free (path);
+			}
 			if (module) {
 				gpointer get_type;
 				if (g_module_symbol (module, "gnome_print__driver_get_type", &get_type)) {
-					ctx = gnome_print_context_create (get_type, cfgnode);
+					ctx = gnome_print_context_create (get_type, config);
 				} else {
 					g_warning ("Missing gnome_print__driver_get_type in %s\n", modulename);
 					g_module_close (module);
@@ -545,7 +544,6 @@ gnome_print_context_new (GnomePrintConfig *config)
 			} else {
 				g_warning ("Cannot open module: %s\n", modulename);
 			}
-			g_free (path);
 			g_free (modulename);
 		} else {
 			g_warning ("Unknown driver: %s", drivername);
@@ -558,7 +556,7 @@ gnome_print_context_new (GnomePrintConfig *config)
 }
 
 static GnomePrintContext *
-gnome_print_context_create (gpointer get_type, GPANode *config)
+gnome_print_context_create (gpointer get_type, GnomePrintConfig *config)
 {
 	GnomePrintContext *ctx;
 	GType (* driver_get_type) (void);
