@@ -84,9 +84,11 @@ gp_gc_new (void)
 	GPGC * gc;
 	GPCtx * ctx;
 
+	ctx = gp_ctx_new ();
+	g_return_val_if_fail (ctx != NULL, NULL);
+
 	gc = g_new (GPGC, 1);
 	gc->refcount = 1;
-	ctx = gp_ctx_new ();
 	gc->ctx = g_slist_prepend (NULL, ctx);
 
 	return gc;
@@ -127,6 +129,7 @@ gp_gc_reset (GPGC * gc)
 	}
 
 	ctx = gp_ctx_new ();
+	g_return_if_fail (ctx != NULL);
 
 	gc->ctx = g_slist_prepend (NULL, ctx);
 }
@@ -149,15 +152,11 @@ gp_gc_gsave (GPGC * gc)
 gint
 gp_gc_grestore (GPGC * gc)
 {
-	GSList *tmp;
-
 	g_return_val_if_fail (gc != NULL, -1);
 	g_return_val_if_fail (gc->ctx->next != NULL, -1);
 
-	tmp = gc->ctx;
-	gp_ctx_destroy (tmp->data);
-	gc->ctx = tmp->next;
-	g_slist_free_1 (tmp);
+	gp_ctx_destroy ((GPCtx *) gc->ctx->data);
+	gc->ctx = g_slist_remove (gc->ctx, gc->ctx->data);
 
 	return 0;
 }
@@ -582,16 +581,40 @@ gint
 gp_gc_moveto (GPGC * gc, gdouble x, gdouble y)
 {
 	GPCtx * ctx;
-	ArtPoint p;
+	ArtPoint p, q;
+	gdouble a[6];
 
 	g_return_val_if_fail (gc != NULL, 1);
 
 	ctx = (GPCtx *) gc->ctx->data;
 
+	memcpy (a, ctx->ctm, 6 * sizeof (gdouble));
 	p.x = x;
 	p.y = y;
-	art_affine_point (&ctx->currentpoint, &p, ctx->ctm);
-
+#if 0
+	g_print ("ABC %g %g\n", p.x, p.y);
+	g_print ("ABD %g %g %g %g %g %g\n", ctx->ctm[0], ctx->ctm[1], ctx->ctm[2], ctx->ctm[3], ctx->ctm[4], ctx->ctm[5]);
+#endif
+	art_affine_point (&q, &p, a);
+#if 0
+	g_print ("ABE %g %g %g %g\n", q.x, q.y, p.x, p.y);
+#endif
+	ctx->currentpoint = q;
+#if 0
+	g_print ("ABF %g %g %g %g\n", ctx->currentpoint.x, ctx->currentpoint.y, q.x, q.y);
+	g_print ("ABG %g %g %g %g %g %g\n", ctx->ctm[0], ctx->ctm[1], ctx->ctm[2], ctx->ctm[3], ctx->ctm[4], ctx->ctm[5]);
+	g_print ("ABC %g %g\n", p.x, p.y);
+	g_print ("ABD %g %g %g %g %g %g\n", ctx->ctm[0], ctx->ctm[1], ctx->ctm[2], ctx->ctm[3], ctx->ctm[4], ctx->ctm[5]);
+#endif
+	art_affine_point (&q, &p, a);
+#if 0
+	g_print ("ABE %g %g %g %g\n", q.x, q.y, p.x, p.y);
+#endif
+	ctx->currentpoint = q;
+#if 0
+	g_print ("ABF %g %g %g %g\n", ctx->currentpoint.x, ctx->currentpoint.y, q.x, q.y);
+	g_print ("ABG %g %g %g %g %g %g\n", ctx->ctm[0], ctx->ctm[1], ctx->ctm[2], ctx->ctm[3], ctx->ctm[4], ctx->ctm[5]);
+#endif
 	gp_path_moveto (ctx->currentpath, ctx->currentpoint.x, ctx->currentpoint.y);
 
 	return 0;
