@@ -46,7 +46,7 @@ static gboolean gpa_printer_verify (GPANode *node);
 static guchar *gpa_printer_get_value (GPANode *node);
 static GPANode *gpa_printer_get_child (GPANode *node, GPANode *ref);
 static GPANode *gpa_printer_lookup (GPANode *node, const guchar *path);
-static void gpa_printer_modified (GPANode *node);
+static void gpa_printer_modified (GPANode *node, guint flags);
 
 static GPANode *gpa_printer_new_from_file (const gchar *filename);
 
@@ -127,7 +127,7 @@ gpa_printer_verify (GPANode *node)
 
 	printer = GPA_PRINTER (node);
 
-	if (!node->id) return FALSE;
+	if (GPA_NODE_ID_EXISTS (node)) return FALSE;
 
 	if (!printer->name) return FALSE;
 	if (!gpa_node_verify (printer->name)) return FALSE;
@@ -146,7 +146,7 @@ gpa_printer_get_value (GPANode *node)
 
 	printer = GPA_PRINTER (node);
 
-	if (node->id) return g_strdup (node->id);
+	if (GPA_NODE_ID_EXISTS (node)) return g_strdup (GPA_NODE_ID (node));
 
 	return NULL;
 }
@@ -195,20 +195,20 @@ gpa_printer_lookup (GPANode *node, const guchar *path)
 }
 
 static void
-gpa_printer_modified (GPANode *node)
+gpa_printer_modified (GPANode *node, guint flags)
 {
 	GPAPrinter *printer;
 
 	printer = GPA_PRINTER (node);
 
 	if (printer->name && (GPA_NODE_FLAGS (printer->name) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (printer->name);
+		gpa_node_emit_modified (printer->name, 0);
 	}
 	if (printer->model && (GPA_NODE_FLAGS (printer->model) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (printer->model);
+		gpa_node_emit_modified (printer->model, 0);
 	}
 	if (printer->settings && (GPA_NODE_FLAGS (printer->settings) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (GPA_NODE (printer->settings));
+		gpa_node_emit_modified (GPA_NODE (printer->settings), 0);
 	}
 }
 
@@ -326,7 +326,7 @@ gpa_printer_new_from_tree (xmlNodePtr tree)
 
 	xmlFree (xmlid);
 
-	if (printer) g_hash_table_insert (iddict, GPA_NODE_ID (printer), printer);
+	if (printer) g_hash_table_insert (iddict, (gpointer) GPA_NODE_ID (printer), printer);
 
 	return (GPANode *) printer;
 }
@@ -447,7 +447,7 @@ gpa_printer_get_by_id (const guchar *id)
 		GPANode *child;
 		for (child = printers->children; child != NULL; child = child->next) {
 			g_assert (GPA_IS_PRINTER (child));
-			if (child->id && !strcmp (id, child->id)) {
+			if (GPA_NODE_ID_COMPARE (child, id)) {
 				gpa_node_ref (child);
 				return child;
 			}
@@ -491,7 +491,7 @@ gpa_printer_new_from_model (GPAModel *model, const guchar *name)
 
 	g_return_val_if_fail (!g_hash_table_lookup (namedict, name), NULL);
 
-	id = gpa_id_new (GPA_NODE (model)->id);
+	id = gpa_id_new (GPA_NODE_ID (model));
 	printer = (GPAPrinter *) gpa_node_new (GPA_TYPE_PRINTER, id);
 	g_free (id);
 
@@ -533,19 +533,19 @@ gpa_printer_save (GPAPrinter *printer)
 	doc = xmlNewDoc ("1.0");
 	root = xmlNewDocNode (doc, NULL, "Printer", NULL);
 	xmlSetProp (root, "Version", "1.0");
-	xmlSetProp (root, "Id", GPA_NODE (printer)->id);
+	xmlSetProp (root, "Id", GPA_NODE_ID (printer));
 	xmlDocSetRootElement (doc, root);
 
 	xmln = xmlNewChild (root, NULL, "Name", GPA_VALUE (printer->name)->value);
 
-	xmln = xmlNewChild (root, NULL, "Model", GPA_REFERENCE (printer->model)->ref->id);
+	xmln = xmlNewChild (root, NULL, "Model", GPA_NODE_ID (GPA_REFERENCE (printer->model)->ref));
 
 	for (child = printer->settings->children; child != NULL; child = child->next) {
 		xmln = gpa_settings_write (doc, child);
 		if (xmln) xmlAddChild (root, xmln);
 	}
 
-	filename = g_strdup_printf ("%s/.gnome/printers/%s.printer", g_get_home_dir (), GPA_NODE (printer)->id);
+	filename = g_strdup_printf ("%s/.gnome/printers/%s.printer", g_get_home_dir (), GPA_NODE_ID (printer));
 	xmlSaveFile (filename, doc);
 	g_free (filename);
 
