@@ -44,7 +44,7 @@ static gboolean gpa_settings_verify (GPANode *node);
 static guchar *gpa_settings_get_value (GPANode *node);
 static GPANode *gpa_settings_get_child (GPANode *node, GPANode *ref);
 static GPANode *gpa_settings_lookup (GPANode *node, const guchar *path);
-static void gpa_settings_modified (GPANode *node);
+static void gpa_settings_modified (GPANode *node, guint flags);
 
 static GPANodeClass *parent_class = NULL;
 
@@ -122,7 +122,7 @@ gpa_settings_duplicate (GPANode *node)
 
 	settings = GPA_SETTINGS (node);
 
-	new = (GPASettings *) gpa_node_new (GPA_TYPE_SETTINGS, node->id);
+	new = (GPASettings *) gpa_node_new (GPA_TYPE_SETTINGS, GPA_NODE_ID (node));
 
 	if (settings->name) {
 		new->name = gpa_node_attach (GPA_NODE (new), gpa_node_duplicate (settings->name));
@@ -216,8 +216,10 @@ gpa_settings_lookup (GPANode *node, const guchar *path)
 	}
 
 	for (child = settings->keys; child != NULL; child = child->next) {
+		const guchar *cid;
 		g_assert (GPA_IS_KEY (child));
-		if (child->id && strlen (child->id) == len && !strncmp (child->id, path, len)) {
+		cid = GPA_NODE_ID (child);
+		if (cid && strlen (cid) == len && !strncmp (cid, path, len)) {
 			if (!next) {
 				gpa_node_ref (child);
 				return child;
@@ -231,7 +233,7 @@ gpa_settings_lookup (GPANode *node, const guchar *path)
 }
 
 static void
-gpa_settings_modified (GPANode *node)
+gpa_settings_modified (GPANode *node, guint flags)
 {
 	GPASettings *settings;
 	GPANode *child;
@@ -239,10 +241,10 @@ gpa_settings_modified (GPANode *node)
 	settings = GPA_SETTINGS (node);
 
 	if (settings->name && GPA_NODE_FLAGS (settings->name) & GPA_MODIFIED_FLAG) {
-		gpa_node_emit_modified (settings->name);
+		gpa_node_emit_modified (settings->name, 0);
 	}
 	if (settings->model && GPA_NODE_FLAGS (settings->model) & GPA_MODIFIED_FLAG) {
-		gpa_node_emit_modified (settings->model);
+		gpa_node_emit_modified (settings->model, 0);
 	}
 
 	child = settings->keys;
@@ -251,7 +253,7 @@ gpa_settings_modified (GPANode *node)
 		next = child->next;
 		if (GPA_NODE_FLAGS (child) & GPA_MODIFIED_FLAG) {
 			gpa_node_ref (child);
-			gpa_node_emit_modified (child);
+			gpa_node_emit_modified (child, 0);
 			gpa_node_unref (child);
 		}
 		child = next;
@@ -368,7 +370,7 @@ gpa_settings_new_from_model_and_tree (GPANode *model, xmlNodePtr tree)
 			if (keyid) {
 				GPANode *key;
 				for (key = settings->keys; key != NULL; key = key->next) {
-					if (!strcmp (keyid, key->id)) {
+					if (GPA_NODE_ID_COMPARE (key, keyid)) {
 						gpa_key_merge_from_tree (key, xmlc);
 						break;
 					}
@@ -397,7 +399,7 @@ gpa_settings_write (xmlDocPtr doc, GPANode *node)
 	settings = GPA_SETTINGS (node);
 
 	xmln = xmlNewDocNode (doc, NULL, "Settings", NULL);
-	xmlSetProp (xmln, "Id", node->id);
+	xmlSetProp (xmln, "Id", GPA_NODE_ID (node));
 
 	xmlc = xmlNewChild (xmln, NULL, "Name", GPA_VALUE (settings->name)->value);
 
@@ -444,7 +446,7 @@ gpa_settings_copy (GPASettings *dst, GPASettings *src)
 	while (sl) {
 		GSList *l;
 		for (l = dl; l != NULL; l = l->next) {
-			if (!strcmp (GPA_NODE (l->data)->id, GPA_NODE (sl->data)->id)) {
+			if (GPA_NODE (l->data)->qid && (GPA_NODE (l->data)->qid == GPA_NODE (sl->data)->qid)) {
 				/* We are in original too */
 				child = GPA_NODE (l->data);
 				dl = g_slist_remove (dl, l->data);
@@ -470,7 +472,7 @@ gpa_settings_copy (GPASettings *dst, GPASettings *src)
 		dl = g_slist_remove (dl, dl->data);
 	}
 
-	gpa_node_request_modified (GPA_NODE (dst));
+	gpa_node_request_modified (GPA_NODE (dst), 0);
 
 	return TRUE;
 }
